<?php

namespace Drupal\pathauto_entity_parent;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Psr\Log\LoggerInterface;

/**
 * Storage helper class.
 */
class StorageHelper {

  /**
   * Config factory interface.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * Entity type manager interface.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Logger interface.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * The config factory.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerChannelFactory
   *   Logger channel factory.
   */
  public function __construct(ConfigFactoryInterface $configFactory, EntityTypeManagerInterface $entityTypeManager, LoggerChannelFactoryInterface $loggerChannelFactory) {
    $this->configFactory     = $configFactory;
    $this->entityTypeManager = $entityTypeManager;
    $this->logger            = $loggerChannelFactory->get('pathauto_entity_parent');
  }

  /**
   * When a bundle is unchecked, we remove the value of the field in the nodes.
   *
   * @param array $bundles
   *   Bundles.
   */
  public function removeValueFromBundles(array $bundles): void {
    try {
      $storage  = $this->entityTypeManager->getStorage('node');
      $results  = $storage->getQuery()
        ->condition('type', $bundles, 'IN')
        ->condition('pathauto_entity_parent', '', '<>')
        ->execute();
      $entities = $storage->loadMultiple($results);
      foreach ($entities as $entity) {
        $entity->set('pathauto_entity_parent', NULL);
        $entity->save();
      }
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException | EntityStorageException $e) {
      $this->logger->error($e->getMessage());
    }
  }

}
